@@ -1,4 +1,4 @@
-import os, copy, pydicom
+import os, copy, pydicom, inspect
 import numpy as np
 from pydicom.tag import Tag
 import SimpleITK as sitk
@@ -79,9 +79,8 @@ def contour_worker(A):
 
 
 def worker_def(A):
-    q, Contour_Names, associations, desc, final_out_dict = A
-    base_class = DicomReaderWriter(get_images_mask=True, associations=associations,
-                                   Contour_Names=Contour_Names, desc=desc, get_dose_output=True)
+    q, key_dict, final_out_dict = A
+    base_class = DicomReaderWriter(**key_dict)
     while True:
         item = q.get()
         if item is None:
@@ -93,10 +92,11 @@ def worker_def(A):
                 base_class.Make_Contour_From_directory(PathDicom=path)
                 base_class.set_iteration(iteration)
                 base_class.write_images_annotations(out_path)
-                final_out_dict['MRN'].append(base_class.ds.PatientID)
-                final_out_dict['Iteration'].append(iteration)
-                final_out_dict['Path'].append(path)
-                final_out_dict['Folder'].append('')
+                if iteration not in final_out_dict['Iteration']:
+                    final_out_dict['MRN'].append(base_class.ds.PatientID)
+                    final_out_dict['Iteration'].append(iteration)
+                    final_out_dict['Path'].append(path)
+                    final_out_dict['Folder'].append('')
             except:
                 print('failed on {}'.format(path))
             q.task_done()
@@ -128,7 +128,7 @@ class Point_Output_Maker_Class(object):
 
 class DicomReaderWriter:
     def __init__(self, rewrite_RT_file=False, delete_previous_rois=True, Contour_Names=None,
-                 template_dir=None, get_images_mask=True, arg_max=True, create_new_RT=True,
+                 template_dir=None, get_images_mask=True, arg_max=True, create_new_RT=True, require_all_contours=True,
                  associations={},desc='',iteration=0, get_dose_output=False, flip_axes=(False, False, False), **kwargs):
         '''
         :param rewrite_RT_file: Boolean, should we re-write the RT structure
@@ -137,6 +137,7 @@ class DicomReaderWriter:
         :param template_dir: default to None, specifies path to template RT structure
         :param get_images_mask: boolean, load the images and mask
         :param arg_max: perform argmax on the mask
+        :param require_all_contours: Boolean, require all contours present when making nifti files?
         :param associations: dictionary of associations {'liver_bma_program_4': 'liver'}
         :param desc: description information to add to .nii files
         :param iteration: what iteration for writing .nii files
@@ -145,6 +146,7 @@ class DicomReaderWriter:
         :param kwargs:
         '''
         self.get_dose_output = get_dose_output
+        self.require_all_contours = require_all_contours
         self.flip_axes = flip_axes
         self.create_new_RT = create_new_RT
         self.associations = associations
@@ -274,7 +276,6 @@ class DicomReaderWriter:
                 elif modality.lower().find('struct') != -1:
                     self.RTs_in_case[lstRSFile] = []
 
-        self.all_contours_exist = False
         self.rois_in_case = []
         self.all_RTs.update(self.RTs_in_case)
         if len(self.RTs_in_case.keys()) > 0:
@@ -296,7 +297,10 @@ class DicomReaderWriter:
             for key in final_out_dict.keys():
                 for index in data[key]:
                     final_out_dict[key].append(data[key][index])
-        A = [q, self.Contour_Names, self.associations, self.desciption, final_out_dict]
+        key_dict = {'get_images_mask': True, 'associations': self.associations, 'arg_max': self.arg_max,
+                    'require_all_contours': self.require_all_contours, 'Contour_Names': self.Contour_Names,
+                    'desc': self.desciption, 'get_dose_output': True}
+        A = [q, key_dict, final_out_dict]
         threads = []
         for worker in range(thread_count):
             t = Thread(target=worker_def, args=(A,))
@@ -384,7 +388,8 @@ class DicomReaderWriter:
                 if found_rois[ROI_Name]['Roi_Number'] in self.structure_references:
                     index = self.structure_references[found_rois[ROI_Name]['Roi_Number']]
                     mask = self.Contours_to_mask(index)
-                    self.mask[..., self.Contour_Names.index(ROI_Name) + 1][mask == 1] = 1
+                    self.mask[..., self.Contour_Names.index(ROI_Name) + 1][mask == 1] += 1
+                    self.mask[self.mask > 1] = 1
         if self.flip_axes[0]:
             self.mask = self.mask[:, :, ::-1, ...]
         if self.flip_axes[1]:
@@ -728,9 +733,9 @@ class DicomReaderWriter:
                 print('Lacking {} in {}'.format(roi, PathDicom))
                 print('Found {}'.format(self.rois_in_case))
                 self.all_contours_exist = False
-                break
-        if PathDicom not in self.paths_with_contours and self.all_contours_exist:
-            self.paths_with_contours.append(PathDicom) # Add the path that has the contours
+        if PathDicom not in self.paths_with_contours:
+            if self.all_contours_exist or not self.require_all_contours:
+                self.paths_with_contours.append(PathDicom) # Add the path that has the contours
         return None
 
     def rewrite_RT(self, lstRSFile=None):
